import { sleep } from '../utils/eventLoop';
import { type StringifiableValue, createSerializer } from '../utils/serializer';
import type { ClearableKeyValueStorage } from './types';

export const createKeyValueStorage = <TValue extends StringifiableValue>(
    latencyMs: number,
): ClearableKeyValueStorage<string, TValue> => {
    const withSimulatedLatency = async <TResult>(
        f: () => Promise<TResult>,
    ): Promise<TResult> => {
        // TODO: do some CPU load here, to make performance similar to real DBs, for performance testing
        await sleep(latencyMs);
        // TODO: do some additional CPU load here
        return f();
    };

    const serializer = createSerializer<TValue>();
    const storage = new Map<string, string>();
    return {
        get: (key) =>
            withSimulatedLatency(() =>
                Promise.resolve(
                    storage.has(key)
                        ? {
                              found: true as const,
                              // `storage.get(key)` is guaranteed to return value, because we just checked that it's there
                              // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                              value: serializer.parse(storage.get(key)!),
                          }
                        : {
                              found: false as const,
                          },
                ),
            ),
        set: (key, value) =>
            withSimulatedLatency(() =>
                Promise.resolve(
                    void storage.set(key, serializer.stringify(value)),
                ),
            ),
        clear: () =>
            // eslint-disable-next-line @typescript-eslint/no-confusing-void-expression
            withSimulatedLatency(() => Promise.resolve(storage.clear())),
    };
};
