import { CarrierType } from '../types';
import { createKeyValueStorage } from './inMemoryDB';
import { PackageInfo, PackagesRepository } from './types';

const samplePackages = (
    [
        [
            'TN12345677',
            'DHL',
            'Street 1, 10115 Berlin, Germany',
            'Hamburger Str. 273A, 38114 Braunschweig, Germany',
            'Laptop',
            1,
            800,
            'LP123',
            'in-transit',
        ],
        [
            'TN12345678',
            'DHL',
            'Street 1, 10115 Berlin, Germany',
            'Carrer de Tapioles, 47, 08004 Barcelona, Spain',
            'Mouse',
            1,
            25,
            'MO456',
            'in-transit',
        ],
        [
            'TN12345679',
            'UPS',
            'Street 2, 20144 Hamburg, Germany',
            'Dumlupınar Sk. No:5, 34710 Kadıköy/İstanbul, Türkiye',
            'Monitor',
            2,
            200,
            'MT789',
            'inbound-scan',
        ],
        [
            'TN12345680',
            'DPD',
            'Street 3, 80331 Munich, Germany',
            'Street 5, 28013 Madrid, Spain',
            'Keyboard',
            1,
            50,
            'KB012',
            'delivery',
        ],
        [
            'TN12345681',
            'DPD',
            'Street 3, 80331 Munich, Germany',
            'Street 5, 28013 Madrid, Spain',
            'Mouse',
            1,
            25,
            'MO456',
            'delivery',
        ],
        [
            'TN12345682',
            'FedEx',
            'Street 4, 50667 Cologne, Germany',
            'Street 9, 1016 Amsterdam, Netherlands',
            'Laptop',
            1,
            900,
            'LP345',
            'transit',
        ],
        [
            'TN12345683',
            'FedEx',
            'Street 4, 50667 Cologne, Germany',
            'Street 9, 1016 Amsterdam, Netherlands',
            'Headphones',
            1,
            100,
            'HP678',
            'transit',
        ],
        [
            'TN12345684',
            'GLS',
            'Street 5, 70173 Stuttgart, Germany',
            'Street 15, 1050 Copenhagen, Denmark',
            'Smartphone',
            1,
            500,
            'SP901',
            'scanned',
        ],
        [
            'TN12345685',
            'GLS',
            'Street 5, 70173 Stuttgart, Germany',
            'Street 15, 1050 Copenhagen, Denmark',
            'Charger',
            1,
            20,
            'CH234',
            'scanned',
        ],
    ] as const
).map(
    ([
        trackingNumber,
        carrier,
        senderAddress,
        receiverAddress,
        articleName,
        articleQuantity,
        articlePrice,
        SKU,
        status,
    ]) => ({
        trackingNumber,
        carrier,
        senderAddress,
        receiverAddress,
        articleName,
        articleQuantity,
        articlePrice,
        SKU,
        status,
    }),
);

const getKey = (carrier: CarrierType, trackingNumber: string) =>
    `${carrier}/${trackingNumber}`;

export const createSamplePackagesRepository =
    async (): Promise<PackagesRepository> => {
        const storage = createKeyValueStorage<PackageInfo>(50);

        await Promise.all(
            samplePackages.map((samplePackage) =>
                storage.set(
                    getKey(samplePackage.carrier, samplePackage.trackingNumber),
                    samplePackage,
                ),
            ),
        );

        return {
            getPackage: (carrier: CarrierType, trackingNumber: string) =>
                storage.get(getKey(carrier, trackingNumber)),
        };
    };
