import { CarrierType, PackageStatusType } from '../types';

type RowInfo<TValue> = { found: true; value: TValue } | { found: false };

export type KeyValueStorage<TKey, TValue> = {
    get(key: TKey): Promise<RowInfo<TValue>>;
    set(key: TKey, value: TValue): Promise<void>;
};

export type ClearableKeyValueStorage<TKey, TValue> = KeyValueStorage<
    TKey,
    TValue
> & {
    clear(): Promise<void>;
};

export type PackageInfo = {
    trackingNumber: string;
    carrier: CarrierType;
    senderAddress: string;
    receiverAddress: string;
    articleName: string;
    articleQuantity: number;
    articlePrice: number;
    SKU: string;
    status: PackageStatusType;
};

export type PackagesRepository = {
    getPackage(
        this: void,
        carrier: CarrierType,
        trackingNumber: string,
    ): Promise<RowInfo<PackageInfo>>;
};
